//
//  ContentView.swift
//  Demo
//
//  Created by Kustiawanto Halim on 25/10/21.
//

import SwiftUI

struct ContentView: View {
    @ObservedObject var viewModel = ContentViewViewModel()
    
    var body: some View {
        VStack {
            Spacer()
            
            Text("Guess it!")
                .font(.title)
            
            Text("\(viewModel.givenValue)  +  __\(viewModel.guessValue)__  =  \(viewModel.expectValue)")
                .frame(width: UIScreen.main.bounds.width, height: 65)
                .background(viewModel.bgColor)
                .foregroundColor(Color.white)
                .padding()
            
            HStack {
                Spacer()
                
                Button { viewModel.minusButtonPressed() } label: {
                    Text("-")
                        .font(.title)
                        .frame(width: 50, height: 50)
                        .foregroundColor(Color.white)
                        .background(viewModel.minusButtonBackground())
                        .clipShape(Circle())
                }.disabled(viewModel.minusButtonLimit())
                
                Spacer()
                
                Button { viewModel.plusButtonPressed() } label: {
                    Text("+")
                        .font(.title)
                        .frame(width: 50, height: 50)
                        .foregroundColor(Color.white)
                        .background(viewModel.plusButtonBackground())
                        .clipShape(Circle())
                }.disabled(viewModel.plusButtonLimit())
                
                Spacer()
                
            }.padding()
            
            Spacer()
            
            Button("Reset") { viewModel.resetButtonPressed() }.padding()
        }.onAppear {
            viewModel.checkResult()
        }
    }
}

class ContentViewViewModel: ObservableObject {
    @Published var givenValue: Int = Int.random(in: 1...10)
    @Published var guessValue: Int = 0
    @Published var expectValue: Int = Int.random(in: 1...10)
    
    @Published var bgColor: Color?
}

extension ContentViewViewModel {
    func checkResult() {
        if (expectValue == guessValue + givenValue) {
            bgColor = Color.green
        } else {
            bgColor = Color.yellow
        }
    }
    
    func minusButtonLimit() -> Bool {
        return guessValue < -10
    }
    
    func minusButtonBackground() -> Color {
        return minusButtonLimit() ? Color.gray : Color.blue
    }
    
    func minusButtonPressed() {
        guessValue = guessValue - 1
        checkResult()
    }
    
    func plusButtonLimit() -> Bool {
        return guessValue > 10
    }
    
    func plusButtonBackground() -> Color {
        return plusButtonLimit() ? Color.gray : Color.blue
    }
    
    func plusButtonPressed() {
        guessValue = guessValue + 1
        checkResult()
    }
    
    func resetButtonPressed() {
        guessValue = 0
        givenValue = Int.random(in: 0...10)
        expectValue = Int.random(in: 0...10)
        checkResult()
    }
}

struct ContentView_Previews: PreviewProvider {
    static var previews: some View {
        ContentView()
    }
}
