//
//  DemoTests.swift
//  DemoTests
//
//  Created by Kustiawanto Halim on 25/10/21.
//

import XCTest
import SwiftUI
@testable import Demo

class DemoTests: XCTestCase {
    
    var sut: ContentViewViewModel!

    override func setUpWithError() throws {
        try super.setUpWithError()
        sut = ContentViewViewModel()
    }

    override func tearDownWithError() throws {
        sut = nil
        try super.tearDownWithError()
    }

    func testMinusButtonLimitReturnFalseAndBlueBackground() throws {
        // Given
        sut.guessValue = Int.random(in: -10 ..< 0)
        
        // When
        let result = sut.minusButtonLimit()
        let background = sut.minusButtonBackground()
        
        // Then
        XCTAssertFalse(result)
        XCTAssertEqual(background, Color.blue)
    }

    func testMinusButtonLimitReturnTrueAndGrayBackground() throws {
        // Given
        sut.guessValue = Int.random(in: -20 ..< -10)
        
        // When
        let result = sut.minusButtonLimit()
        let background = sut.minusButtonBackground()
        
        // Then
        XCTAssertTrue(result)
        XCTAssertEqual(background, Color.gray)
    }
    
    func testPlusButtonLimitReturnFalseAndBlueBackground() throws {
        // Given
        sut.guessValue = Int.random(in: 0 ..< 10)
        
        // When
        let result = sut.plusButtonLimit()
        let background = sut.plusButtonBackground()
        
        // Then
        XCTAssertFalse(result)
        XCTAssertEqual(background, Color.blue)
    }
    
    func testPlusButtonLimitReturnTrueAndGrayBackground() throws {
        // Given
        sut.guessValue = Int.random(in: 11 ..< 20)
        
        // When
        let result = sut.plusButtonLimit()
        let background = sut.plusButtonBackground()
        
        // Then
        XCTAssertTrue(result)
        XCTAssertEqual(background, Color.gray)
    }
    
    func testPlusButtonPressed() throws {
        // Given
        let latestValue = Int.random(in: 0 ..< 10)
        sut.guessValue = latestValue
        
        // When
        sut.plusButtonPressed()
        
        // Then
        XCTAssertEqual(sut.guessValue, latestValue + 1)
    }
    
    func testMinusButtonPressed() throws {
        // Given
        let latestValue = Int.random(in: 0 ..< 10)
        sut.guessValue = latestValue
        
        // When
        sut.minusButtonPressed()
        
        // Then
        XCTAssertEqual(sut.guessValue, latestValue - 1)
    }
    
    func testResetButtonPressed() throws {
        // Given
        sut.guessValue = Int.random(in: 1 ..< 10)
        sut.givenValue = Int.random(in: 11 ... 20)
        sut.expectValue = Int.random(in: 11 ..< 20)
        let newRandomRange = 0 ... 10
        
        // When
        sut.resetButtonPressed()
        
        // Then
        XCTAssertEqual(sut.guessValue, 0)
        XCTAssertTrue(newRandomRange ~= sut.givenValue)
        XCTAssertTrue(newRandomRange ~= sut.expectValue)
    }
    
    func testCheckResultReturnGreenColor() throws {
        // Given
        sut.guessValue = 4
        sut.givenValue = 3
        sut.expectValue = 7
        
        // When
        sut.checkResult()
        
        // Then
        XCTAssertEqual(sut.bgColor, Color.green)
    }
    
    func testCheckResultReturnYellowColor() throws {
        // Given
        sut.guessValue = Int.random(in: 5 ... 10)
        sut.givenValue = 3
        sut.expectValue = 7
        
        // When
        sut.checkResult()
        
        // Then
        XCTAssertEqual(sut.bgColor, Color.yellow)
    }
}
